{
 DXSprSrc - Sprite Sources unit.
            Requieres DirectDraw.pas from
            DirectX 8.0 headers for Delphi.

            You can find those headers here;
            http://www.delphi-jedi.org/DelphiGraphics/
}

unit DXSprSrc;

// *******************************************************************

interface

uses Windows, Classes, DirectDraw, SysUtils, DXDraw, DXGDI;

type // TDDSprSrc - "Sprite Source".
     TDDSprSrc = record
                   Filename : pChar;
                   Data     : IDirectDrawSurface7;
                   Width    : longint;
                   Height   : longint;
                   Rect     : TRect;
                 end;
     pTDDSprSrc = ^TDDSprSrc;

// Sprite Source functions.
procedure DDResetSprSrc(var _sprSrc: TDDSprSrc);
function  DDCreateSprSrc(_filename: string): TDDSprSrc;
function  DDRestoreSprSrc(_spr: TDDSprSrc): boolean;
procedure DDFreeSprSrc(var _sprSrc: TDDSprSrc);
function  DDValidSprSrc(_spr: TDDSprSrc): boolean;

// Other functions.
function DDSetColorKey(_surf: IDirectDrawSurface7;
                       _rgb: TColorRef): boolean;

// *******************************************************************

implementation

// *******************************************************************
// Name : DDResetSprSrc.
// Desc : Resets a TDDSprSrc variable.
// *******************************************************************

procedure DDResetSprSrc(var _sprSrc: TDDSprSrc);

begin
  FillChar(_sprSrc, SizeOf(_sprSrc), 0);
end;

// *******************************************************************
// Name : DDCreateSprSrcFromBMP.
// Desc : Create a "Sprite Source" from a Bitmap.
//        (Do not use this function directly.)
// *******************************************************************

function DDCreateSprSrcFromBMP(_hBMP: hBitmap): TDDSprSrc;

var _tBMP : tBitmap;
    _ddsd : TDDSurfaceDesc2;
    _spr : IDirectDrawSurface7;
    _hr : hResult;

begin
  DDResetSprSrc(Result);

  // Get dimension of the BMP.
  GetObject(_hBMP, sizeof(TBITMAP), @_tBMP);
  Result.Width  := _tBMP.bmWidth;
  Result.Height := _tBMP.bmHeight;
  Result.Rect   := Rect(0, 0, _tBMP.bmWidth, _tBMP.bmHeight);

  // Set up surface description.
  with _ddsd do
    begin
      FillChar(_ddsd, SizeOf(_ddsd), #0);
      dwSize  := sizeof(_ddsd);
      dwFlags := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT;
      ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN;
      dwWidth  := Result.Width;
      dwHeight := Result.Height;
    end;

  // Create the surface.
  _hr := DD7.CreateSurface(_ddsd, _spr, nil);
  if failed(_hr) then exit;

  // Copy then BMP.
  if not DDDrawBitmap(_hBMP, 0, 0, Result.Width, Result.Height,
                             0, 0, Result.Width, Result.Height, _spr)
    then begin _spr := NIL; exit; end;

  // Return the surface.
  Result.Data := _spr;
end;

// *******************************************************************
// Name : DDFreeSprSrc.
// Desc : Frees a Sprite Source.
// *******************************************************************

procedure DDFreeSprSrc(var _sprSrc: TDDSprSrc);

begin
  StrDispose(_sprSrc.Filename);
  DDResetSprSrc(_sprSrc);
end;

// *******************************************************************
// Name : DDCreateSprSrc.
// Desc : Create a "Sprite Source" from a BMP file.
// *******************************************************************

function DDCreateSprSrc(_filename: string): TDDSprSrc;

var _hbm : hBitmap;

begin
  DDResetSprSrc(Result);

  // Load the BMP.
  _hbm := LoadImage(0, PChar(_filename), IMAGE_BITMAP, 0, 0,
                    LR_LOADFROMFILE or LR_CREATEDIBSECTION);
  if _hbm = 0 then exit;

  // Create the Sprite Source from the BMP.
  Result := DDCreateSprSrcFromBMP(_hbm);

  // Free the BMP.
  DeleteObject(_hbm);

  // Allocate memory and store the BMP filename.
  if Result.Data <> NIL then
    begin
      Result.Filename := StrNew(pChar(_filename));
      if Result.Filename = NIL then
        begin Result.Data := NIL; exit; end;
    end;
end;

// *******************************************************************
// Name : DDRestoreSprSrc.
// Desc : Restores the "Sprite Source" surface and reloads the BMP.
// *******************************************************************

function DDRestoreSprSrc(_spr: TDDSprSrc): boolean;

var _hbm : hBitmap;

begin
  Result := false;

  if _spr.Data <> NIL then
    begin
      _spr.Data._Restore; // Restore the surface.
      if _spr.Filename <> NIL then
        begin
          // Try to reload the bitmap.
          _hbm := LoadImage(0, _spr.Filename, IMAGE_BITMAP, 0, 0,
                            LR_LOADFROMFILE or LR_CREATEDIBSECTION);
          if _hbm = 0 then exit;

          // Draw the BMP.
          Result := DDDrawBitmap(_hbm, 0, 0, _spr.Width, _spr.Height,
                                 0, 0, _spr.Width, _spr.Height,
                                 _spr.Data);

          // Free the BMP.
          DeleteObject(_hbm);
        end;
    end;
end;

// *******************************************************************
// Name : DDValidSprSrc.
// Desc : Checks if a Sprite Source is valid.
// *******************************************************************

function DDValidSprSrc(_spr: TDDSprSrc): boolean;

begin
  Result := (_spr.Data <> NIL);
end;

// *******************************************************************
// Name : DDSetColorKey
// Desc : Sets the color key using DXGDI's DDColorMatch function.
//        (Use only under non paletized video modes.)
// *******************************************************************

function DDSetColorKey(_surf: IDirectDrawSurface7;
                       _rgb: TColorRef): boolean;

var _color : TColorRef;
    _ddck : TDDColorKey;

begin
  Result := false;

  // Get the color key for this surface.
  _color := DDColorMatch(_surf, _rgb);
  if _color = CLR_INVALID then exit;

  // Set the color key range.
  _ddck.dwColorSpaceLowValue  := _color;
  _ddck.dwColorSpaceHighValue := _color;

  // Set the color key range to the surface.
  Result := not Failed(_surf.SetColorKey(DDCKEY_SRCBLT, @_ddck));
end;

// *******************************************************************

end.
